/* This file is part of Lcrzo.
*/

int lcrzo_dir_value_pos(const char *dirname,
			lcrzo_int32 pos,
			lcrzo_string *pfilename)
#if defined LCRZODEF_SYSTEM_Unix
{ DIR *thisdir;
  struct dirent *file;
  int ret;
  lcrzo_string filename;
  lcrzo_string cleandirname;
  lcrzo_int32 count, pospos;

  lcrzo_er(lcrzo_dir_count(dirname, &count));
  lcrzo_er(lcrzo_priv_manageelempos(count, pos, 0, NULL, &pospos, NULL,NULL));

  /*open the directory*/
  lcrzo_er(lcrzo_priv_dirname_removelastslash(dirname, &cleandirname));
  thisdir=opendir(cleandirname);
  lcrzo_string_free(cleandirname);
  if ( thisdir==NULL ) return(LCRZO_ERR_FUOPENDIR);

  file=readdir(thisdir);
  count=0;
  while (file!=NULL)
  { lcrzo_efr(lcrzo_string_initm_text(file->d_name, &filename),
	      closedir(thisdir););
    if ( strcmp(filename, ".") && strcmp(filename, "..") )
    { /*printf("%s\n", filename);*/
      count++;
      if (count==pospos) 
      { if ( pfilename == NULL ) lcrzo_string_free(filename);
	else *pfilename=filename;
        break;
      }
    }
    lcrzo_string_free(filename);
    file=readdir(thisdir);
  }
  ret=closedir(thisdir);
 
  return(LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{ struct _finddata_t fileinfos;
  lcrzo_int32 count, pospos;
  lcrzo_string filename;
  lcrzo_string cleandirname;
  long hFile;
  int ret;

  lcrzo_er(lcrzo_dir_count(dirname, &count));
  lcrzo_er(lcrzo_priv_manageelempos(count, pos, 0, NULL, &pospos, NULL,NULL));

  /* verify if directory exist */
  lcrzo_er(lcrzo_priv_dirname_removelastslash(dirname, &cleandirname));
  ret=lcrzo_string_initm_text(cleandirname, &filename);
  lcrzo_string_free(cleandirname);
  if (ret!=LCRZO_ERR_OK) return(ret);
  lcrzo_er(lcrzo_string_appendm_text("\\*", &filename));
  hFile=_findfirst(filename, &fileinfos);
  lcrzo_string_free(filename);
  if (hFile==-1) return(LCRZO_ERR_FUFINDFIRST);

  /* count files */
  count=0;
  do 
  { if ( strcmp(fileinfos.name, ".") && strcmp(fileinfos.name, "..") )
    { count++;
      if (count==pospos) 
      { lcrzo_er(lcrzo_string_initm_text(fileinfos.name, pfilename));
        break;
      }
    }
    ret=_findnext(hFile, &fileinfos);
  }
  while (ret==0);
  lcrzo_err_purge();

  /* close */
  ret=_findclose( hFile );
  if (ret==-1) return(LCRZO_ERR_FUFINDCLOSE);

  return(LCRZO_ERR_OK);
}
#else
 #error "Unknown value for LCRZODEF_SYSTEM"
#endif

